from setuptools import setup

setup(name='distributionsgeneral',
      version='0.1',
      description='Gaussian and Binomial distributionsgeneral',
      packages=['distributionsgeneral'],
      author='Amandeep Mander',
      zip_safe=False)